/*******************************************************************************
 * Copyright 2009 Seapine Software, Inc. All rights reserved.
 * 
 * IMPORTANT:  This Software is supplied to you by Seapine Software, Inc. (Seapine)
 * in consideration of your agreement to the following terms, and your use, installation,
 * modification or redistribution of this Software constitutes acceptance of these terms.
 * If you do not agree with these terms, please do not use, install, modify or redistribute
 * this Software.
 * 
 * In consideration of your agreement to abide by the following terms, and subject to these
 * terms, Seapine grants you a personal, non-exclusive license, under Seapine copyrights in
 * this original Seapine software (the Software), to use, reproduce, modify and redistribute
 * the Software, with or without modifications, in source and/or binary forms; provided that
 * if you redistribute the Software in its entirety and without modifications, you must retain
 * this notice and the following text and disclaimers in all such redistributions of the Software.
 * Neither the name, trademarks, service marks or logos of Seapine, Inc. may be used to endorse
 * or promote products derived from the Software without specific prior written permission from
 * Seapine.  Except as expressly stated in this notice, no other rights or licenses, express or
 * implied, are granted by Seapine herein, including but not limited to any patent rights that
 * may be infringed by your derivative works or by other works in which the Software may be
 * incorporated.
 * 
 * The Software is provided by Seapine on an "AS IS" basis.  SEAPINE MAKES NO WARRANTIES,
 * EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED WARRANTIES OF NON-INFRINGEMENT,
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, REGARDING THE SOFTWARE OR ITS USE AND
 * OPERATION ALONE OR IN COMBINATION WITH YOUR PRODUCTS. 
 * 
 * IN NO EVENT SHALL SEAPINE BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) ARISING IN ANY WAY OUT OF THE USE,
 * REPRODUCTION, MODIFICATION AND/OR DISTRIBUTION OF THE SOFTWARE, HOWEVER CAUSED AND WHETHER
 * UNDER THEORY OF CONTRACT, TORT (INCLUDING NEGLIGENCE), STRICT LIABILITY OR OTHERWISE, EVEN
 * IF SEAPINE HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************/
using System;
using System.Collections;
using System.Data;
using System.Text;
using Seapine.SurroundSCM.API;
using System.Windows.Forms;

namespace Rollback_Changelist
{
    class Engine
    {
        
        
        SSCMContext context;
        Changelist[] list = new Changelist[0];

        ArrayList FileList = new ArrayList();

        //Variables gathered from environment variables.  These are used to show changelists for the 
        //current user, current branch.
        String scmBranch = "";
        String scmMainline = "";
        String scmCurUser = "";

        //Variables used for connecting to Surround SCM
        String scmUser = "";
        String scmPassword = "";
        String scmServer = "";
        uint scmPort = 0;

        public Engine(CMConfig config)
        {
            scmUser = config.getUserNameFromFile();
            scmPassword = config.getPasswordFromFile();
            scmServer = config.getSSCMServerFromFile();
            scmPort = Convert.ToUInt16(config.getSSCMPortFromFile());

            scmBranch = Environment.GetEnvironmentVariable("SSCM_BRANCH");
            scmMainline = Environment.GetEnvironmentVariable("SSCM_MAINLINE");
            scmCurUser = Environment.GetEnvironmentVariable("SSCM_USER");

            
        }
        //This method gets the committed changelists for user.  For each one, it gets the list of files that 
        //were committed as part of the changelist.  For each file, it looks for those that "Check in" was the 
        //action related to the changelists.  For each of these files, it also gets the current version of the file
        //If they are not the same, a flag is set to warn the user that other actions have taken place since the changelist 
        //was committed.
        public DataTable GetChangeslists()
        {
            //Declare data table that will contain the changelists
            DataTable changeTable = new DataTable();
            //initialize SSCMContext for connecting to SSCM and connect
            context = new SSCMContext(); 
            SSCMResult result = SSCMAPI.Connect(scmServer, scmPort, scmUser, scmPassword, out context);
            SetContext();
            if (result.IsOk())
            {
                result = SSCMAPI.GetCommittedChangelistsForUser(context, scmCurUser,0, out list);
                if (list.Length > 0)
                {
                    //Build the table and columns
                    DataColumn[] changeColumns = new DataColumn[5];
                    changeColumns[0] = new DataColumn();
                    changeColumns[0].ColumnName = "Changelist Name";
                    changeColumns[1] = new DataColumn();
                    changeColumns[1].ColumnName = "Username";
                    changeColumns[2] = new DataColumn();
                    changeColumns[2].ColumnName = "Date Committed";
                    changeColumns[3] = new DataColumn();
                    changeColumns[3].ColumnName = "Version Mismatch";
                    changeColumns[4] = new DataColumn();
                    changeColumns[4].ColumnName = "Rolled Back";
                    for (int i = 0; i < changeColumns.Length; i++)
                    {
                        changeTable.Columns.Add(changeColumns[i]);
                    }
                    //Let's go through the list of committed changelists and fill the table
                    for (int i = 0; i < list.Length; i++)
                    {
                        //Let's define a var to keep track of how many files are in the changelist whose action was a check in
                        int fileCount = 0;
                        //Define new row
                        DataRow changeRow = changeTable.NewRow();
                        changeRow[0] = list[i].GetName();
                        changeRow[1] = list[i].GetUsername();
                        changeRow[2] = list[i].GetTimestamp();
                        //We need a flag to see if any file in the list has been updated after current changelist
                        bool mismatch = false;
                        //Now we build the file ArrayList
                        //Get files from changelist
                        ChangelistFile[] listFiles = new ChangelistFile[0];
                        listFiles = list[i].GetFiles();
                        //Now we go through the file list for the current changelist and look for check in actions
                        for (int j = 0; j < listFiles.Length; j++)
                        {
                            if (listFiles[j].GetAction().Equals("Check In"))
                            {
                                String[] fileDetails = new String[7];
                                fileDetails[0] = list[i].GetName();
                                fileDetails[1] = listFiles[j].GetRepository();
                                fileDetails[2] = listFiles[j].GetName();
                                fileDetails[3] = listFiles[j].GetAction();
                                fileDetails[4] = listFiles[j].GetVersion().ToString();
                                SSCMFile fileProps = new SSCMFile();
                                //We also would like the current version
                                result = SSCMAPI.FileProperties(context, listFiles[j].GetRepository(), listFiles[j].GetName(), out fileProps);
                                if (result.IsOk())
                                {
                                    fileDetails[5] = fileProps.GetVersion().ToString();
                                }
                                //Rollbacks by this app will place the changelist name in the comments of the rollback action
                                //If we find the current changelist name in the comments for a rollback, then this changelist has already
                                //been rolled back!
                                FileHistoryItem[] fileHistory = new FileHistoryItem[0];
                                result = SSCMAPI.FileHistory(context, listFiles[j].GetRepository(), listFiles[j].GetName(), false, false, FileAction.RollbackFile, out fileHistory);
                                if (result.IsOk())
                                {
                                    for (int k = 0; k < fileHistory.Length; k++)
                                    {
                                        if (fileHistory[k].GetComment().Contains(list[i].GetName()))
                                        {
                                            changeRow[4] = "Yes";
                                        }
                                    }
                                }
                                //Changelist version
                                uint chVersion = listFiles[j].GetVersion();
                                //Current version
                                uint curVersion = fileProps.GetVersion();
                                //Set flag if they are not the same
                                if (chVersion != curVersion)
                                {
                                    mismatch = true;
                                    fileDetails[6] = "true";

                                }
                                else
                                {
                                    fileDetails[6] = "false";
                                }
                                FileList.Add(fileDetails);
                                fileCount++;
                            }
                        }
                        if (mismatch == false)
                        {
                            changeRow[3] = "false";
                        }
                        else
                        {
                            changeRow[3] = "true";
                        }
                        if (fileCount > 0)
                        {
                            changeTable.Rows.Add(changeRow);
                        }

                    }
                }                
                SSCMAPI.Disconnect(context);
            }
            else
            {
                MessageBox.Show("Unable to Log in","Rollback Changelist");
                Application.Exit();
            }
            
            return changeTable;
        }
        private void SetContext()
        {
            context.SetBranch(scmBranch);
            context.SetMainline(scmMainline);

        }
        public DataTable GetFiles(String changelistName)
        {                        
            DataTable fileTable = new DataTable();
            DataColumn[] fileColumns = new DataColumn[6];
            fileColumns[0] = new DataColumn();
            fileColumns[0].ColumnName = "Repository";
            fileColumns[1] = new DataColumn();
            fileColumns[1].ColumnName = "File Name";
            fileColumns[2] = new DataColumn();
            fileColumns[2].ColumnName = "Action";
            fileColumns[3] = new DataColumn();
            fileColumns[3].ColumnName = "Version";
            fileColumns[4] = new DataColumn();
            fileColumns[4].ColumnName = "Current Version";
            fileColumns[5] = new DataColumn();
            fileColumns[5].ColumnName = "Version Mismatch";
            

            for (int i = 0; i < fileColumns.Length; i++)
            {
                fileTable.Columns.Add(fileColumns[i]);
            }
            for (int i = 0; i < FileList.Count; i++)
            {
                string[] fileDetails = new string[5];
                fileDetails = (string[])FileList[i];
                if (fileDetails[0] == changelistName)
                {
                    DataRow fileRow = fileTable.NewRow();
                    fileRow[0] = fileDetails[1];
                    fileRow[1] = fileDetails[2];
                    fileRow[2] = fileDetails[3];
                    fileRow[3] = fileDetails[4];
                    fileRow[4] = fileDetails[5];
                    fileRow[5] = fileDetails[6];
                    fileTable.Rows.Add(fileRow);
                }
            }
            
            return fileTable;
        }
        public String RollBack(String changelistName)
        {
            String rollResult = "";
            SSCMResult result = SSCMAPI.Connect(scmServer, scmPort, scmUser, scmPassword, out context);
            if (result.IsOk())
            {
                for (int i = 0; i < FileList.Count; i++)
                {
                    string[] fileDetails = new string[6];
                    fileDetails = (string[])FileList[i];
                    if (fileDetails[0] == changelistName)
                    {
                        SetContext();
                        uint chVersion = Convert.ToUInt16(fileDetails[4]);
                        uint rollBackVersion = chVersion - 1;
                        SSCMResult rresult = SSCMAPI.RollbackFile(context, fileDetails[1], fileDetails[2], "API Rollback for Changelist " + fileDetails[0] + ".", rollBackVersion);
                        if (!result.IsOk())
                        {
                            rollResult = "Error while attempting rollback";
                        }
                    }
                }
            }
            else
            {
                rollResult = "There was a problem logging in.";
            }
            SSCMAPI.Disconnect(context);
            return rollResult;
        }

    }
}
