/*******************************************************************************
 * Copyright 2009 Seapine Software, Inc. All rights reserved.
 * 
 * IMPORTANT:  This Software is supplied to you by Seapine Software, Inc. (Seapine)
 * in consideration of your agreement to the following terms, and your use, installation,
 * modification or redistribution of this Software constitutes acceptance of these terms.
 * If you do not agree with these terms, please do not use, install, modify or redistribute
 * this Software.
 * 
 * In consideration of your agreement to abide by the following terms, and subject to these
 * terms, Seapine grants you a personal, non-exclusive license, under Seapine copyrights in
 * this original Seapine software (the Software), to use, reproduce, modify and redistribute
 * the Software, with or without modifications, in source and/or binary forms; provided that
 * if you redistribute the Software in its entirety and without modifications, you must retain
 * this notice and the following text and disclaimers in all such redistributions of the Software.
 * Neither the name, trademarks, service marks or logos of Seapine, Inc. may be used to endorse
 * or promote products derived from the Software without specific prior written permission from
 * Seapine.  Except as expressly stated in this notice, no other rights or licenses, express or
 * implied, are granted by Seapine herein, including but not limited to any patent rights that
 * may be infringed by your derivative works or by other works in which the Software may be
 * incorporated.
 * 
 * The Software is provided by Seapine on an "AS IS" basis.  SEAPINE MAKES NO WARRANTIES,
 * EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED WARRANTIES OF NON-INFRINGEMENT,
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, REGARDING THE SOFTWARE OR ITS USE AND
 * OPERATION ALONE OR IN COMBINATION WITH YOUR PRODUCTS. 
 * 
 * IN NO EVENT SHALL SEAPINE BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) ARISING IN ANY WAY OUT OF THE USE,
 * REPRODUCTION, MODIFICATION AND/OR DISTRIBUTION OF THE SOFTWARE, HOWEVER CAUSED AND WHETHER
 * UNDER THEORY OF CONTRACT, TORT (INCLUDING NEGLIGENCE), STRICT LIABILITY OR OTHERWISE, EVEN
 * IF SEAPINE HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************/
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace Rollback_Changelist
{
    public partial class frmMain : Form
    {
        public frmMain()
        {
            InitializeComponent();
        }
        Engine scmEngine;
        private void frmMain_Load(object sender, EventArgs e)
        {
            try
            {
                CMConfig config = new CMConfig();
                config.SetPath(Application.StartupPath);
                if (config.Exists())
                {
                    scmEngine = new Engine(config);
                    DataTable changeTable = scmEngine.GetChangeslists();
                    if (changeTable.Rows.Count > 0)
                    {
                        dgvCList.DataSource = changeTable;
                        dgvCList.Columns[0].Width = 400;
                        dgvCList.Columns[2].Width = 210;
                        //Value on the fourth column not being used right now, so let's just hide it!
                        dgvCList.Columns[3].Visible = false;

                    }
                    else
                    {
                        MessageBox.Show("No committed changelists were returned.\n The user must not have committed changelists in the selected branch." + System.Environment.GetEnvironmentVariable("SSCM_USER"));
                        Application.Exit();
                    }
                }
                else
                {
                    MessageBox.Show("Configuration file not found!", "Rollback Changelist");
                    Application.Exit();
                }
            }
            catch (Exception fcc)           
            {
                MessageBox.Show("General Exception reported.\nError reported:\n" + fcc.Message,"Rollback Changelist");
                
                
            }
           
        }

        private void dgvCList_RowEnter(object sender, DataGridViewCellEventArgs e)
        {
            if (dgvCList.Rows.Count > 0)
            {
                if (dgvCList.Columns.Count > 0)
                {
                    if (dgvCList.SelectedRows.Count > 0)
                    {
                        DataTable fileTable = scmEngine.GetFiles(dgvCList.SelectedRows[0].Cells[0].Value.ToString());
                        dgvFiles.DataSource = fileTable;
                        
                        dgvFiles.Columns[0].Width = 250;
                        dgvFiles.Columns[1].Width = 250;
                        //The sixth value is an internal flag, don't need to show it to the user
                        dgvFiles.Columns[5].Visible = false;
                        dgvFiles.SelectedRows[0].Selected = false;
                    }
                }
            }

        }

        private void btnRollback_Click(object sender, EventArgs e)
        {
            try
            {
                String rollResult = "";
                if (dgvCList.SelectedRows[0].Cells[3].Value.ToString().Equals("true"))
                {
                    DialogResult result = MessageBox.Show("You are about to rollback a changelist whose changes are not the most current.\nAny changes made by the changelist and after it will be lost.\nAre you sure?", "Rollback Changelist", MessageBoxButtons.YesNo);
                    if (result == DialogResult.Yes)
                    {
                        if (dgvCList.SelectedRows[0].Cells[4].Value.Equals("Yes"))
                        {
                            result = MessageBox.Show("It appears that this changelist has already been rolled back.\nContinue?", "Rollback Changelist", MessageBoxButtons.YesNo);
                            if (result == DialogResult.Yes)
                            {
                                result = MessageBox.Show("You are about to rollback changes. This rollback is not atomic and there is no undo available.\nDo you want to continue?", "Rollabck Changelist", MessageBoxButtons.YesNo);
                                if (result == DialogResult.Yes)
                                {
                                    rollResult = scmEngine.RollBack(dgvCList.SelectedRows[0].Cells[0].Value.ToString());
                                    if (rollResult.Equals(""))
                                    {
                                        MessageBox.Show("File Rolled Back", "Rollback Changelist");
                                    }
                                    else
                                    {
                                        MessageBox.Show("File was not Rolled Back." + rollResult);
                                    }
                                }
                            }
                        }
                        else
                        {
                            result = MessageBox.Show("You are about to rollback changes. This rollback is not atomic and there is no undo available.\nDo you want to continue?", "Rollabck Changelist", MessageBoxButtons.YesNo);
                            if (result == DialogResult.Yes)
                            {
                                rollResult = scmEngine.RollBack(dgvCList.SelectedRows[0].Cells[0].Value.ToString());
                                if (rollResult.Equals(""))
                                {
                                    MessageBox.Show("File Rolled Back", "Rollback Changelist");
                                }
                                else
                                {
                                    MessageBox.Show("File was not Rolled Back." + rollResult);
                                }
                            }
                        }
                    }

                }
                else
                {
                    DialogResult result = MessageBox.Show("You are about to rollback changes. This rollback is not atomic and there is no undo available.\nDo you want to continue?", "Rollabck Changelist", MessageBoxButtons.YesNo);
                    if (result == DialogResult.Yes)
                    {
                        rollResult = scmEngine.RollBack(dgvCList.SelectedRows[0].Cells[0].Value.ToString());
                        if (rollResult.Equals(""))
                        {
                            MessageBox.Show("File Rolled Back", "Rollback Changelist");
                        }
                        else
                        {
                            MessageBox.Show("File was not Rolled Back." + rollResult);
                        }
                    }
                }
            }
            catch (Exception fcc)
            {
                MessageBox.Show("General Exception reported.\nError reported:\n" + fcc.Message, "Rollback Changelist");
            }
            
        }

        private void btnClose_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }
    }
}